/*
* This file is part of Bielefeld Sensor Actuator Interface (BonSAI).
*
* Copyright(c) Frederic Siepmann
* http://opensource.cit-ec.de/projects/bonsai
*
* This file may be licensed under the terms of of the
* GNU Lesser General Public License Version 3 (the ``LGPL''),
* or (at your option) any later version.
*
* Software distributed under the License is distributed
* on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
* express or implied. See the LGPL for the specific language
* governing rights and limitations.
*
* You should have received a copy of the LGPL along with this
* program. If not, go to http://www.gnu.org/licenses/lgpl.html
* or write to the Free Software Foundation, Inc.,
* 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
*
* The development of this software was supported by the
* Excellence Cluster EXC 277 Cognitive Interaction Technology.
* The Excellence Cluster EXC 277 is a grant of the Deutsche
* Forschungsgemeinschaft (DFG) in the context of the German
* Excellence Initiative.
*
* Contributors: Leon Ziegler, Johannes Wienke, Norman Köster, Torben Töniges,
*               Annika Rothert, Alexander Baumann, Marcel Dulisch, Maikel Linke,
*               Marian Pohling, Lukas Twardon, Lukas Kettenbach
*/
package de.unibi.airobots.bonsai.actuators;

import java.io.IOException;

import de.unibi.airobots.bonsai.core.Actuator;

/**
 * Interface for arena entering and localization actuators.
 * 
 * @author ltwardon
 */
public interface EnterArenaAndLocalizeActuator extends Actuator {

    /**
     * Let the robot enter the arena and do a mini-exploration in order to localize itself on the map
     * 
     * @param meters
     *      How far the robot should enter the arena
     * 
     * @throws IOException
     *             communication error
     */
    void enterArenaAndLocalize(double meters) throws IOException;
    
    /**
     * Let the robot enter the arena and do a mini-exploration in order to localize itself on the map
     * 
     * @param meters
     *      How far the robot should enter the arena
     * @param priorKnowledgeX
     *      Prior Knowledge about where the robot might be when starting to localize itself (X-Coord)
     * @param priorKnowledgeY
     *      Prior Knowledge about where the robot might be when starting to localize itself (Y-Coord)
     * @param priorKnowledgeDiv
     *      Prior Knowledge about where the robot might be when starting to localize itself (Divergence from X, Y)
     * 
     * @throws IOException
     *             communication error
     */
    void enterArenaAndLocalize(double meters, double priorKnowledgeX, double priorKnowledgeY, double priorKnowledgeDiv) throws IOException;

    /**
     * Wait until the robot has entered the arena and finished the exploration/localization
     * 
     * @throws IOException
     *             communication error
     */
    void waitUntilLocalizationFinished() throws IOException;
    
}
