package de.unibi.airobots.bonsai.skills.helper;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import nu.xom.Element;
import nu.xom.Elements;
import nu.xom.ParsingException;
import nu.xom.ValidityException;

import org.apache.log4j.Logger;
import org.xml.sax.SAXException;

import de.unibi.airobots.bonsai.util.helper.annotation.MapArea;
import de.unibi.airobots.bonsai.util.helper.annotation.MapRegion;
import de.unibi.airobots.bonsai.util.helper.annotation.MapArea.MapAreaType;
import de.unibi.airobots.bonsai.util.xml.ParameterMissingException;
import de.unibi.airobots.bonsai.util.xml.XMLDecoderUtil;
import de.unibi.airobots.bonsai.util.xml.XMLEncoderUtil;
import de.unibi.airobots.btl.data.navigation.NavigationGoalData;
import de.unibi.airobots.btl.data.navigation.NavigationGoalData.GoalType;
import de.unibi.airobots.btl.data.navigation.PositionData;

/**
 * 
 * @author jhelbach
 * 
 */
public class MapAnnotationHelper {

	public final static String ANNOTATION_FILE_PATH = "/tmp/log/annotations_"
			+ System.getenv("USER") + ".xml";

	private Element root;

	private String annotationFilePath;

	/**
	 * Logger
	 */
	private Logger logger = Logger.getLogger(this.getClass());

	private Map<String, MapArea> areas;

	public MapAnnotationHelper(String annotationFilePath) throws Exception {

		if (!new File(annotationFilePath).exists())
			throw new Exception("File not found " + annotationFilePath);

		this.annotationFilePath = annotationFilePath;

		logger.debug("load file " + annotationFilePath);
		root = XMLDecoderUtil.loadXML(annotationFilePath);
		logger.debug("root element " + root + " childcount "
				+ root.getChildCount());

		this.areas = new HashMap<String, MapArea>();

		Elements areaNodes = root.getChildElements();

		for (int i = 0; i < areaNodes.size(); i++) {

			MapArea area = XMLDecoderUtil.getMapArea(areaNodes.get(i));

			logger.debug("add area " + area + " area name "
					+ area.getArenaName());
			areas.put(area.getType().name(), area);
			logger.debug("areas add " + areas.size());
		}
	}

	public MapAnnotationHelper() throws Exception {

		if (!new File(ANNOTATION_FILE_PATH).exists())
			throw new Exception("File not found " + ANNOTATION_FILE_PATH);

		this.annotationFilePath = ANNOTATION_FILE_PATH;

		logger.debug("load file " + ANNOTATION_FILE_PATH);
		root = XMLDecoderUtil.loadXML(ANNOTATION_FILE_PATH);
		logger.debug("root element " + root + " childcount "
				+ root.getChildCount());

		this.areas = new HashMap<String, MapArea>();

		Elements areaNodes = root.getChildElements();

		for (int i = 0; i < areaNodes.size(); i++) {

			MapArea area = XMLDecoderUtil.getMapArea(areaNodes.get(i));

			logger.debug("add area " + area + " area name "
					+ area.getArenaName());
			areas.put(area.getType().name(), area);
			logger.debug("areas add " + areas.size());
		}
	}

	public PositionData getRegioPositionsByLabel(String label) throws Exception {

		logger.debug("get position by label :" + label);
		List<PositionData> pd = getPositionForRegion(label, 1);
		logger.debug("PositionData " + pd);

		if (pd.size() != 1) {
			return null;
		}

		return pd.get(0);
	}

	public NavigationGoalData getRegioNavigationGoalByLabel(String label)
			throws Exception {

		List<PositionData> pd = getPositionForRegion(label, 1);

		if (pd.size() != 1) {
			return null;
		}

		NavigationGoalData navRet = new NavigationGoalData(pd.get(0));
		navRet.setType(GoalType.GLOBAL);

		return navRet;
	}

	public List<PositionData> getRegioPositionsByGroup(String group)
			throws Exception {

		return getPositionForRegion(group, 2);
	}

	/**
	 * 
	 * if task == 1 find position data for label if task == 2 find position data
	 * for group
	 * 
	 * @param label
	 * @param task
	 * @return
	 * @throws Exception
	 */
	private List<PositionData> getPositionForRegion(String label, int task)
			throws Exception {

		Collection<MapArea> mapAreas = areas.values();
		logger.debug("map areas count " + mapAreas.size());

		List<PositionData> positionDatas = new ArrayList<PositionData>();

		for (MapArea a : mapAreas) {

			for (MapRegion r : a.getRegions()) {

				String forCompare = "";

				switch (task) {
				case 1:
					forCompare = r.getLabel();
					break;
				case 2:
					forCompare = r.getGroup();
					break;

				}

				logger.debug("label " + label);
				logger.debug("found label/group " + forCompare);

				if (forCompare.equals(label)) {
					positionDatas.add(r.getPositionData());
					logger.debug("position datas count " + positionDatas.size());
				}
			}
		}

		return positionDatas;
	}

	public MapRegion getRegionOfAreaTyp(MapAreaType areaType, String label)
			throws Exception {
		return getRegion(annotationFilePath, areaType, label);

	}

	public Map<String, MapArea> getAreas() {
		return new HashMap<String, MapArea>(areas);
	}

	public static MapRegion getRegion(MapAreaType areaType, String label)
			throws ValidityException, SAXException, ParsingException,
			IOException, NumberFormatException, ParameterMissingException {
		return getRegion(ANNOTATION_FILE_PATH, areaType, label);
	}

	public static MapRegion getRegion(String annotationFile,
			MapAreaType areaType, String label) throws ValidityException,
			SAXException, ParsingException, IOException, NumberFormatException,
			ParameterMissingException {

		Element root = XMLDecoderUtil.loadXML(annotationFile);
		Elements childs = root.getChildElements();

		MapArea area = null;

		for (int i = 0; i < childs.size(); i++) {

			Element child = childs.get(i);

			if (child.getAttribute(XMLEncoderUtil.TYPE_ATTRNAME).getValue()
					.equals(areaType.name())) {
				area = XMLDecoderUtil.getMapArea(child);
			}

		}

		if (area != null) {
			return area.getRegionByLabel(label);
		}

		return null;
	}

}
