/*
 * This file is part of Bielefeld Sensor Actuator Interface (BonSAI).
 *
 * Copyright(c) Frederic Siepmann
 * http://opensource.cit-ec.de/projects/bonsai
 *
 * This file may be licensed under the terms of of the
 * GNU Lesser General Public License Version 3 (the ``LGPL''),
 * or (at your option) any later version.
 *
 * Software distributed under the License is distributed
 * on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
 * express or implied. See the LGPL for the specific language
 * governing rights and limitations.
 *
 * You should have received a copy of the LGPL along with this
 * program. If not, go to http://www.gnu.org/licenses/lgpl.html
 * or write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * The development of this software was supported by the
 * Excellence Cluster EXC 277 Cognitive Interaction Technology.
 * The Excellence Cluster EXC 277 is a grant of the Deutsche
 * Forschungsgemeinschaft (DFG) in the context of the German
 * Excellence Initiative.
 *
 * Contributors: Leon Ziegler, Johannes Wienke, Norman Köster, Torben Töniges,
 *               Annika Rothert, Alexander Baumann, Marcel Dulisch, Maikel Linke,
 *               Marian Pohling, Lukas Twardon, Lukas Kettenbach
 */
package de.unibi.airobots.bonsai.skills.view;

import java.awt.Dimension;
import javax.swing.*;

/**
 * This panel contains a list with all initialized sensors and actuators.
 * 
 * @author lkettenb
 */
public class SensorsAndActuatorsListPanel extends JPanel {
    /*
     * Prefferd size of this panel.
     */
    private static final int PANEL_WIDTH = 250;
    private static final int PANEL_HEIGHT = 300;
    
    private static final String LINE = 
            " - - - - - - - - - - - - - - - - - - - - - ";
    
    
    /**
     * DefaultListModel to control sensor and actuator list.
     */
    private DefaultListModel lSensorsActuatorsModel = new DefaultListModel();
    
    /**
     * JList with all sensors and actuators.
     */
    private JList lSensorsActuators = new JList(lSensorsActuatorsModel);
    
    /**
     * JScrollPane for the JList.
     */
    private JScrollPane scrollPane = new JScrollPane(lSensorsActuators);
    
    /**
     * Convenience constructor.
     */
    public SensorsAndActuatorsListPanel() {
        super();
        setBorder(BorderFactory.createTitledBorder(" Sensors & Actuators "));
        scrollPane.setPreferredSize(new Dimension(PANEL_WIDTH, PANEL_HEIGHT));
        add(scrollPane);
    }
    
    
    /**
     * Update the sensors and actuators list.
     * 
     * @param sensors List with all sensors.
     * @param actuators List with all actuators.
     */
    public void updateSensorsActuatorsList(String[] sensors, 
            String[] actuators) {
        lSensorsActuatorsModel.clear();
        for (String sensor : sensors) {
            lSensorsActuatorsModel.addElement(sensor);
        }
        lSensorsActuatorsModel.addElement(LINE);        
        lSensorsActuators.setSelectedIndex(
                lSensorsActuatorsModel.getSize() - 1);
        
        for (String actuator : actuators) {
            lSensorsActuatorsModel.addElement(actuator);
        }
    }
}
