package de.unibi.airobots.bonsai.util.arm.calibration;

import java.io.IOException;
import java.util.ArrayList;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.xpath.XPath;
import javax.xml.xpath.XPathConstants;
import javax.xml.xpath.XPathExpression;
import javax.xml.xpath.XPathFactory;

import org.w3c.dom.Document;
import org.w3c.dom.NodeList;

import Jama.Matrix;
import de.unibi.airobots.bonsai.actuators.PoseActuatorTobi;
import de.unibi.airobots.bonsai.core.Sensor;
import de.unibi.airobots.bonsai.core.SensorListener;
import de.unibi.airobots.btl.data.geometry.Point3D;
import de.unibi.airobots.btl.data.grasp.KatanaPoseData;
import de.unibi.airobots.btl.units.AngleUnit;
import de.unibi.airobots.btl.units.LengthUnit;

public class CalibrationControl implements SensorListener<Point3D>{

	public static Options options;
	public Sensor<Point3D> chessboardSensor;
	public PoseActuatorTobi poseActuatorTobi;
	public String posesPath;

	public CalibrationControl(Sensor<Point3D> cS, PoseActuatorTobi pAT, String calibrationPoses) {
		
		chessboardSensor = cS;
		chessboardSensor.addSensorListener(this);
		poseActuatorTobi = pAT;
		posesPath = calibrationPoses;
		
		// Initiate calibration movement
		this.calibrationMovement();
	}
	
	/**
	 * Since the control of the Katana arm is vulnerable to 
	 * multiple commands sent in too short time slots, 
	 * it is necessary to wait for a few seconds after each movement.
	 * 3 seconds worked just fine so far, but this depends on both,
	 * the way ToBI and the Katana arm feels.
	 */
	public void waitAfterMovement () {
		long time = System.currentTimeMillis() / 1000L;
		while ((System.currentTimeMillis() / 1000L) < (time + 3L)) {};
	}
	
	/**
	 * Starts the actual movement.
	 */
	public void calibrationMovement () {
		try {
			// Step 1:	Move the gripper to the bottom since the first position are low.
				//poseActuatorTobi.setMovements("grasp_down_init_close");
				//waitAfterMovement();
			//speechManager.sayAccentuated("Please give me the calibration tool!", false);
				poseActuatorTobi.openGripper();
				waitAfterMovement();
				poseActuatorTobi.closeGripperByForce();
				waitAfterMovement();
			
			// Step 2: Generate a set of positions to move to.
				//ArrayList<ArrayList<KatanaPoseData>> poselists = generatePoses();
			ArrayList<KatanaPoseData> poses = readXMLFile(posesPath);

			// Count absolute number of poses for the dimension of the matrices.
			
			int elemCount = 0;
//				for(ArrayList<KatanaPoseData> poses: poselists) {
//					elemCount+= poses.size();
//				}
//				
			
			elemCount = poses.size(); //TODO
			
			Matrix[] pointsA = new Matrix[elemCount];
			Matrix[] pointsB = new Matrix[elemCount];
			
			// Time stamps guarantee that the position of the observed calibration tool 
			// is newer than the time the arm reached its position.
			long armTimestamp = 0;
			long chessboardTimestamp = 0;
//			int listID = 0;
			// Step 3: Go through all poses ad try to reach them.
//			for(ArrayList<KatanaPoseData> poses: poselists) { //FOR SCHLEIFE
				// Since the poses are distributed to multiple pose lists, 
				// we have to go through the individual lists.
				for (int p = 0; p < poses.size(); p++) {
					KatanaPoseData pd = poses.get(p);
					if (pd!=null) {
						System.out.println("Going to x: " + pd.getX(LengthUnit.MILLIMETER) + " y:" + pd.getY(LengthUnit.MILLIMETER) + " z:" + pd.getZ(LengthUnit.MILLIMETER) + 
								" phi:" + pd.getPhi(AngleUnit.RADIAN) + " psi:" + pd.getPsi(AngleUnit.RADIAN) + " theta:" + pd.getTheta(AngleUnit.RADIAN));
						poseActuatorTobi.goTo(pd, true);
						waitAfterMovement();
						armTimestamp = System.currentTimeMillis() / 1000L;
						// The observed position of the calibration tool.
						Point3D calibrationToolPosition;
						do {
							calibrationToolPosition = chessboardSensor.readLast();
							chessboardTimestamp = calibrationToolPosition
							.getTimestamp()
							.getUpdated()
							.getTime()/1000L;
							System.out.println("Waiting for chessboardData newer than " + armTimestamp + " (currently is " + chessboardTimestamp + ")");
						} while (chessboardTimestamp < armTimestamp && calibrationToolPosition != null);
						double[][] aSource = {
								{calibrationToolPosition.getX(LengthUnit.MILLIMETER)},
								{calibrationToolPosition.getY(LengthUnit.MILLIMETER)},
								{calibrationToolPosition.getZ(LengthUnit.MILLIMETER)}
						};
						Matrix a = new Matrix(aSource);
						double[][] bSource = {
								{pd.getX(LengthUnit.MILLIMETER)},
								{pd.getY(LengthUnit.MILLIMETER)},
								{pd.getZ(LengthUnit.MILLIMETER)}
						};
						Matrix b = new Matrix(bSource);
						pointsA[p] = a;
						pointsB[p] = b;
					}
				}
				/* Right now, the first list only contains positions in lower regions.
				 * The second list covers positions in upper regions.
				 * This workaround is a bit dirty since it presumes a maximum of two lists
				 * with the properties named above.
				 */
//				if(listID == 0) {
					/* List of lower positions is completed.
					 * To be able to reach upper position, the arm has to perform a twist.
					 * During this twist, the calibration tool should be removed.
					 */
			//		speechActuator.say("I will now open my gripper.", true);
			//		speechActuator.say("Please remove the chessboard!", false);
//					poseActuatorTobi.openGripper();
//					waitAfterMovement();
//					poseActuatorTobi.setMovements("calibration_twist");
//					waitAfterMovement();
			//		speechActuator.say("Please give me the chessboard!", false);
//					poseActuatorTobi.openGripper();
//					waitAfterMovement();
//					poseActuatorTobi.closeGripperByForce();
//				}
//				if(listID == 1) {
					/* Second and last list is completed.
					 * The calibration tool is released and the arm is
					 * moved to a safe position.
					 */
			//		speechActuator.say("I will now open my gripper.", true);
			//		speechActuator.say("Please remove the chessboard!", false);
					poseActuatorTobi.openGripper();
					waitAfterMovement();
//					poseActuatorTobi.setMovements("go_home_up");
//					waitAfterMovement();
//				}
//				listID++;
//			} //FOR SCHLEIFE
			/* Step 4: Calculate the homography using the arm- and
			 * observed positions. The resulting matrix is saved
			 * in the location given in HomographyCalculation.java
			 */
			HomographyCalculation hc = new HomographyCalculation();
			hc.calculate(pointsA, pointsB);
			
			System.out.println("FERTIG!");
			
		} catch (IOException e) {
			System.err.println("Error moving arm!");
			e.printStackTrace();
		} catch (InterruptedException e) {
			System.err.println("Error reading Data from Kinect");
			e.printStackTrace();
		} finally {
			
		}
	}
	
	/**
	 * Generate a set of positions in a limited interval.
	 * Lower and upper positions are distributed to two different lists.
	 * @return A list containing the two lists of upper and lower positions respectively.
	 */
	@SuppressWarnings("unused")
	private ArrayList<ArrayList<KatanaPoseData>> generatePoses() {
		ArrayList<ArrayList<KatanaPoseData>> list = new ArrayList<ArrayList<KatanaPoseData>>();
		/* Step 1:
		 * 	Lower positions are computed.
		 */
		double phi = 3.087451219558716, psi=-1.567844986915588, theta=1.500036001205444;
		
		// Dimensions of the lower interval.
		int minX = -170, maxX = 170;
		int minY = -100, maxY = 0;
		int minZ = 350, maxZ = 390;
		
		// number of points in the dimensional depending interval.
		int xSteps = 3;
		int ySteps = 2;
		int zSteps = 2;
		
		// range between the points in millimeters is calculated.
		double xStepsize = (maxX - minX) / (xSteps-1);
		double yStepsize = (maxY - minY) / (ySteps-1);
		double zStepsize = (maxZ - minZ) / (zSteps-1);
		
		// set starting position.
		double x=minX, y=minY, z=minZ;
		int xStart = minX;
		int yStart = minY;
		int zStart = minZ;
		
		// list1 will contain the lower positions.
		ArrayList<KatanaPoseData> list1 = new ArrayList<KatanaPoseData>();
		for(int zStep = 0; zStep < zSteps; zStep++) {
			for(int yStep = 0; yStep < ySteps; yStep++) {
				for(int xStep = 0; xStep < xSteps; xStep++) {
					x = Math.round((xStart + xStep * xStepsize) * 100) / 100;
					y = Math.round((yStart + yStep * yStepsize) * 100) / 100;
					z = Math.round((zStart + zStep * zStepsize) * 100) / 100;
					list1.add(new KatanaPoseData(x, y, z, phi, psi, theta,AngleUnit.RADIAN,LengthUnit.MILLIMETER));
				}
				// switch from right-to-left to left-to-right and back after each cycle.
				xStepsize = -xStepsize;
				if (xStart == minX) xStart = maxX; else xStart = minX;
			}
			// switch from top-to-bottom to bottom-to-top and back after each cycle.
			yStepsize = -yStepsize;
			if (yStart == minY) yStart = maxY; else yStart = minY;
		}
		// add list1 to the output list.
		list.add(list1);

		/* Step 3:
		 *  Upper positions are computed similarly.
		 */
		phi = 3.112389802932739;
		psi=1.572011828422546;
		theta=1.59997820854187;
		
		minX = -128; maxX = 128;
		minY = 300; maxY = 350;
		minZ = 280; maxZ = 350;
		
		xSteps = 3;
		ySteps = 2;
		zSteps = 2;
		
		xStepsize = (maxX - minX) / (xSteps-1);
		yStepsize = (maxY - minY) / (ySteps-1);
		zStepsize = (maxZ - minZ) / (zSteps-1);
		
		
		x=minX; y=minY; z=minZ;
		xStart = minX;
		yStart = minY;
		zStart = minZ;
		
		ArrayList<KatanaPoseData> list2 = new ArrayList<KatanaPoseData>();
		for(int zStep = 0; zStep < zSteps; zStep++) {
			for(int yStep = 0; yStep < ySteps; yStep++) {
				for(int xStep = 0; xStep < xSteps; xStep++) {
					x = Math.round((xStart + xStep * xStepsize) * 100) / 100;
					y = Math.round((yStart + yStep * yStepsize) * 100) / 100;
					z = Math.round((zStart + zStep * zStepsize) * 100) / 100;
					list2.add(new KatanaPoseData(x, y, z, phi, psi, theta,AngleUnit.RADIAN,LengthUnit.MILLIMETER));
				}
				xStepsize = -xStepsize;
				if (xStart == minX) xStart = maxX; else xStart = minX;
			}
			yStepsize = -yStepsize;
			if (yStart == minY) yStart = maxY; else yStart = minY;
		}
		list.add(list2);
		return list;
	}

	/**
	 * Reads in the givenXML file and extracts a list of KatanaPositionData.
	 * This function is no longer used, since the positions are now computed.
	 */
	private ArrayList<KatanaPoseData> readXMLFile(String filename) {
		ArrayList<KatanaPoseData> poseList = new ArrayList<KatanaPoseData>();
		try {
			DocumentBuilderFactory domFactory = DocumentBuilderFactory.newInstance();
		    domFactory.setNamespaceAware(true);
		    DocumentBuilder builder = domFactory.newDocumentBuilder();
		    Document doc = builder.parse(filename);
	
			
			XPathFactory factory = XPathFactory.newInstance();
		    XPath xpath = factory.newXPath();
		    XPathExpression expr 
		       = xpath.compile("/record/p");
		     //= xpath.compile("/CALIBRATION/POSES/POSE");
	
		    Object result = expr.evaluate(doc, XPathConstants.NODESET);
		    NodeList nodes = (NodeList) result;
		    for (int i = 0; i < nodes.getLength(); i++) {
		    	double phi = Double.parseDouble(nodes
		    			.item(i)
		    			.getAttributes()
		    			.item(0)
		    			.getNodeValue());
		    	double psi = Double.parseDouble(nodes.item(i).getAttributes().item(1).getNodeValue());
		    	double theta = Double.parseDouble(nodes.item(i).getAttributes().item(2).getNodeValue());
		    	double x = Double.parseDouble(nodes.item(i).getAttributes().item(3).getNodeValue());
		    	double y = Double.parseDouble(nodes.item(i).getAttributes().item(4).getNodeValue());
		    	double z = Double.parseDouble(nodes.item(i).getAttributes().item(5).getNodeValue());
		    	KatanaPoseData kpd = new KatanaPoseData(x, y, z, phi, psi, theta,AngleUnit.RADIAN,LengthUnit.MILLIMETER);
		    	poseList.add(kpd);
		    }
		} catch (Exception e) {
			e.printStackTrace();
		}
		return poseList;
	}

	@Override
	public void newDataAvailable(Point3D newData) {
		// TODO Auto-generated method stub
		
	}
}